#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <util/delay.h>
#include "compass_color.h"

// all on PORTC
#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

// pin functions
#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

#define	BLACK	0x0000
#define	BLUE	0x001F
#define	RED		0xF800
#define	GREEN	0x07E0
#define CYAN	0x07FF
#define MAGENTA	0xF81F
#define YELLOW	0xFFE0
#define WHITE	0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

//extern prog_uchar ten_by_b
unsigned int upscale(unsigned int color);

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

void Delay(uint16 n)
{
	while (n--)
	asm("nop");
}
/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{;}      //0.1ms
}
/*************************************************/
void write_command(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTA=command>>8;

	CLR_WR;
	SET_WR;

	PORTA=command;

	CLR_WR;
	SET_WR;
	SET_CS;
}

/*************************************************/
void write_data(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTA=data>>8;

	CLR_WR;
	SET_WR;

	PORTA=data;

	CLR_WR;
	SET_WR;
	SET_CS;
}
/*************************************************/
void write_data_2(unsigned int data1, unsigned int data2)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTA=data1;

	CLR_WR;
	SET_WR;

	PORTA=data2;

	CLR_WR;
	SET_WR;
	SET_CS;
}
/*************************************************/
void initialization()
{
	CLR_RD;
	CLR_WR;
	SET_CS;
	SET_CD;
	PORTA=0x00;

	CLR_RESET;
	delay(1000);
	SET_RESET;
	delay(5000);

	write_command(0x0001); 
	write_data(0x0100); // Set SS and SM bit
	write_command(0x0002);
	write_data( 0x0700); // Set line inversion
	write_command(0x0003);
	write_data( 0x1030); // Set Write direction
	write_command(0x0004);
	write_data(0x0000); // Set Scaling function off
	write_command(0x0008); 
	write_data(0x0202); // Set BP and FP
	write_command(0x0009); 
	write_data(0x0000); // Set non-display area
	write_command(0x000A); 
	write_data(0x0000); // Frame marker control
	write_command(0x000C); 
	write_data(0x0000); // Set interface control
	write_command(0x000D); 
	write_data(0x0000); // Frame marker Position
	write_command(0x000F); 
	write_data(0x0000); // Set RGB interface 
	 delay(10);
	//--------------- Power On Sequence----------------//
	write_command(0x0010); 
	write_data(0x0000); // Set SAP);BT[3:0]);AP);SLP);STB
	write_command(0x0011); 
	write_data(0x0007); // Set DC1[2:0]);DC0[2:0]);VC
	write_command(0x0012); 
	write_data(0x0000); // Set VREG1OUT voltage
	write_command(0x0013); 
	write_data(0x0000); // Set VCOM AMP voltage
	write_command(0x0007); 
	write_data(0x0001); 
	write_command(0x0007); 
	write_data(0x0020); 
	delay(200);
	write_command(0x0010); 
	write_data(0x1290); // Set SAP);BT[3:0]);AP);SLP);STB
	write_command(0x0011); 
	write_data(0x0221); // Set DC1[2:0]);DC0[2:0]);VC[2:0]
	delay(50);
	write_command(0x0012); 
	write_data(0x008D); // Set VREG1OUT voltaged
	delay(10);
	write_command(0x0013); 
	write_data(0x1A00); // Set VCOM AMP voltage
	write_command(0x0029); 
	write_data(0x0001); // Set VCOMH voltage
	write_command(0x002B); 
	write_data(0x000c); // Set Frame rate.
	delay(50);
	write_command(0x0020); 
	write_data(0x0000); // Set GRAM Horizontal Address
	write_command(0x0021); 
	write_data(0x0000); // Set GRAM Vertical Address
	//---------------Gamma Control  AN----------------------//
	write_command(0x0030); 
	write_data(0x0604);
	write_command(0x0031); 
	write_data(0x0207);
	write_command(0x0032); 
	write_data(0x0202);
	write_command(0x0035); 
	write_data(0x0000);
	write_command(0x0036); 
	write_data(0x0000);
	write_command(0x0037); 
	write_data(0x0005);
	write_command(0x0038); 
	write_data(0x0205);
	write_command(0x0039); 
	write_data(0x0707);
	write_command(0x003C); 
	write_data(0x0000);
	write_command(0x003D); 
	write_data(0x0000);
	//---------------  RAM Address Control ----------------//
	write_command(0x0050); 
	write_data(0x0000); // Set GRAM Horizontal Start Address
	write_command(0x0051); 
	write_data(0x00EF); // Set GRAM Horizontal End Address
	write_command(0x0052); 
	write_data(0x0000); // Set GRAM Vertical Start Address
	write_command(0x0053); 
	write_data(0x013F); // Set GRAM Vertical End Address
	//---------------  Panel Image Control -----------------//
	write_command(0x0060); 
	write_data(0xA700); // Set Gate Scan line
	write_command(0x0061); 
	write_data(0x0001); // Set NDL); VLE); REV
	write_command(0x006A); 
	write_data(0x0000); // Set Scrolling line
	//---------------  Panel Interface Control---------------//
	write_command(0x0090); 
	write_data(0x0010);
	write_command(0x0092); 
	write_data(0x0000);
	//--------------- Display On-------------------------------//
	write_command(0x0007); 
	write_data(0x0133); // Display on
	write_command(0x0022); 
}
/*************************************************/
void Display_Home()
{
	write_command( 0x0020);//Horizontal Address
	write_data( 0x0000);
	write_command( 0x0021);//Vertical Address
	write_data( 0x0000);
}
/*************************************************/
void display_rgb(unsigned int data)
{
	unsigned int i,j;
	Display_Home();

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			write_data(data);
		}
	}
	write_command(0x22);
}
/*************************************************/
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 side_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 320 * 240; // total # of pixels to write

	Display_Home();
	write_command(0x22);
	
	
	j = 0; // byte count
	
	while (1)
	{
		write_data(buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (side_show_flag)
	{
		for (i=0;i<100;i++)	// delay for a while
		Delay(0xf000);

		pics++;					// inc pic number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/
void LCD_test()
{
	unsigned int i,j;
	Display_Home();
	write_command(0x22);

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			if(i>279)write_data(BLACK);
			else if(i>239)write_data(BLUE);
			else if(i>199)write_data(GREEN);
			else if(i>159)write_data(CYAN);
			else if(i>119)write_data(RED);
			else if(i>79)write_data(MAGENTA);
			else if(i>39)write_data(YELLOW);
			else write_data(WHITE);
		}
	}
}
/*************************************************/
void display_compass_24(unsigned maxX, unsigned int maxY)
{
	unsigned int x, y;
	for(y = 0; y < maxY; y++)
	{
		write_command(0x0020);//Horizontal Address
//		write_data(y);
		write_data(0x0000);
		write_command(0x0021);//Vertical Address
//		write_data(0x0000);
		write_data(y);
		write_command(0x0022);
		for(x = 0; x < maxX; x++)
		{
			unsigned char byte1 = pgm_read_byte(&(compass_color_24[y][x]));
//			unsigned char byte2 = pgm_read_byte(&(compass_color_24[y][x]));
			write_data(upscale(byte1));
		}
	}
	write_command(0x22);
}
/*************************************************/
/*
void display_ten_by_ten(void)
{
	unsigned int x, y;
	for(y = 0; y < 10; y++)
	{
		write_command(0x0020);//Horizontal Address
		write_data(0x0000);
		write_command(0x0021);//Vertical Address
		write_data(y);
		write_command(0x0022);
		for(x = 0; x < 10; x++)
		{
//			write_data_2(ten_by_ten[y][x], ten_by_ten[y][x]);
			unsigned char byte = pgm_read_byte(&(ten_by_ten[y][x]));
			write_data(upscale(byte));
		}
	}
}
*/
/*************************************************/
void test_lines(unsigned int maxX, unsigned int maxY)
{
	unsigned int x, y;
	unsigned int offset = 0;
	for(y = 0; y < maxY; y++)
	{
		write_command(0x0020);//Horizontal Address
		write_data(0x0000);
		write_command(0x0021);//Vertical Address
		write_data(y);
		write_command(0x0022);
		for(x = 0; x < maxX; x++)
		{
			if(x == y - offset)
			{
				write_data(BLACK);
			}
			else if(y % 10 == 0)
			{
				write_data(RED);
			}
			else if(x % 10 == 0)
			{
				write_data(BLUE);
			}
			else
			{
				write_data(WHITE);
			}
		}
		if(y == maxX)
		{
			offset = maxX;
		}
	}
}
/*************************************************/
unsigned int upscale(unsigned int color)
{
//BLACK	0x0000
//BLUE	0x001F
//RED		0xF800
//GREEN	0x07E0
//CYAN	0x07FF
//MAGENTA	0xF81F
//YELLOW	0xFFE0
//WHITE	0xFFFF
	unsigned int newColor;

	if(color == 0xFF)	//BLACK
	{
		newColor = 0x0000;	
	}
	else if(color == 0x85 || color == 0x62)	//GREEN
	{
		newColor = 0x07E0;
	}
	else if(color == 0xAC || color == 0x98)	//BLUE
	{
		newColor = 0x001F;
	}
	else if(color == 0xA4 || color == 0xB3)	//RED
	{
		newColor = 0xF800;
	}
	else if(color == 0x24 || color == 0x0E)	//YELLOW
	{
		newColor = 0xFFE0;
	}
	else if(color == 0x00)	//WHITE
	{
		newColor = 0xFFFF;
	}
	else
	{
		newColor = CYAN;
	}

	return newColor;

}
/*************************************************/
int main()
{

    PORTB = 0x20;       // turn on backlight
	PORTC = 0xFE;	    // all pins high
	PORTD = 0xF8;	    // pull-up on switches, all others off

	DDRA  = 0xFF;    	// low data port, all outputs
	DDRB  = 0xFF;    	// all outputs
	DDRC  = 0xFE;	    // all outputs
	DDRD  = 0x0F;   	// VPP and led pins output

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	Delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	Delay(0xffff);

	// Initialize the display
	initialization();

//	LCD_test();

	sd_restart: 	//label for "sd_restart"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	//	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card
	OSCCAL = 0x00;      //run at high speed for normal operation for SD Card


	Delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	while (1)
	{
		test_lines(240, 320);
		Delay(1000);
		display_compass_24(100, 100);
		Delay(1000);
//		Picture_Slide_Show();
//		Delay(500);
	}
	return 0;

}

